import * as chai from 'chai';
var assert = chai.assert;
import * as sinon from 'sinon';
import * as chaiAsPromised from 'chai-as-promised';
import { cloneDeep, assign } from 'lodash';
import gql from 'graphql-tag';
import { print } from 'graphql/language/printer';
import { ApolloLink, Observable } from 'apollo-link-core';
import InMemoryCache, { IntrospectionFragmentMatcher, } from 'apollo-cache-inmemory';
import ApolloClient, { printAST } from '../src';
import subscribeAndCount from './util/subscribeAndCount';
import { withWarning } from './util/wrap';
import { mockSingleLink } from './mocks/mockLinks';
chai.use(chaiAsPromised);
describe('client', function () {
    it('creates query manager lazily', function () {
        var client = new ApolloClient({
            link: ApolloLink.empty(),
            cache: new InMemoryCache(),
        });
        assert.isUndefined(client.queryManager);
        client.initQueryManager();
        assert.isDefined(client.queryManager);
        assert.isDefined(client.queryManager.dataStore.getCache());
    });
    it('can be loaded via require', function () {
        var ApolloClientRequire = require('../src').default;
        var client = new ApolloClientRequire({
            link: ApolloLink.empty(),
            cache: new InMemoryCache(),
        });
        assert.isUndefined(client.queryManager);
        client.initQueryManager();
        assert.isDefined(client.queryManager);
        assert.isDefined(client.queryManager.dataStore.getCache());
    });
    it('can allow passing in a link', function () {
        var link = ApolloLink.empty();
        var client = new ApolloClient({
            link: link,
            cache: new InMemoryCache(),
        });
        assert.instanceOf(client.link, ApolloLink);
    });
    it('should throw an error if query option is missing or not wrapped with a "gql" tag', function () {
        var client = new ApolloClient({
            link: ApolloLink.empty(),
            cache: new InMemoryCache(),
        });
        assert.throws(function () {
            client.query((_a = ["\n          {\n            a\n          }\n        "], _a.raw = ["\n          {\n            a\n          }\n        "], gql(_a)));
            var _a;
        }, 'query option is required. You must specify your GraphQL document in the query option.');
        assert.throws(function () {
            client.query({ query: '{ a }' });
        }, 'You must wrap the query string in a "gql" tag.');
    });
    it('should throw an error if mutation option is missing', function () {
        var client = new ApolloClient({
            link: ApolloLink.empty(),
            cache: new InMemoryCache(),
        });
        assert.throws(function () {
            client.mutate({
                query: (_a = ["\n            {\n              a\n            }\n          "], _a.raw = ["\n            {\n              a\n            }\n          "], gql(_a)),
            });
            var _a;
        }, 'mutation option is required. You must specify your GraphQL document in the mutation option.');
    });
    it('should allow for a single query to take place', function () {
        var query = (_a = ["\n      query people {\n        allPeople(first: 1) {\n          people {\n            name\n            __typename\n          }\n          __typename\n        }\n      }\n    "], _a.raw = ["\n      query people {\n        allPeople(first: 1) {\n          people {\n            name\n            __typename\n          }\n          __typename\n        }\n      }\n    "], gql(_a));
        var data = {
            allPeople: {
                people: [
                    {
                        name: 'Luke Skywalker',
                        __typename: 'Person',
                    },
                ],
                __typename: 'People',
            },
        };
        return clientRoundtrip(query, { data: data });
        var _a;
    });
    it('should allow a single query with an apollo-link enabled network interface', function (done) {
        var query = (_a = ["\n      query people {\n        allPeople(first: 1) {\n          people {\n            name\n            __typename\n          }\n          __typename\n        }\n      }\n    "], _a.raw = ["\n      query people {\n        allPeople(first: 1) {\n          people {\n            name\n            __typename\n          }\n          __typename\n        }\n      }\n    "], gql(_a));
        var data = {
            allPeople: {
                people: [
                    {
                        name: 'Luke Skywalker',
                        __typename: 'Person',
                    },
                ],
                __typename: 'People',
            },
        };
        var variables = { first: 1 };
        var link = ApolloLink.from([function () { return Observable.of({ data: data }); }]);
        var client = new ApolloClient({
            link: link,
            cache: new InMemoryCache({}, { addTypename: false }),
            addTypename: false,
        });
        client.query({ query: query, variables: variables }).then(function (actualResult) {
            assert.deepEqual(actualResult.data, data);
            done();
        });
        var _a;
    });
    it('should allow for a single query with complex default variables to take place', function () {
        var query = (_a = ["\n      query stuff(\n        $test: Input = { key1: [\"value\", \"value2\"], key2: { key3: 4 } }\n      ) {\n        allStuff(test: $test) {\n          people {\n            name\n          }\n        }\n      }\n    "], _a.raw = ["\n      query stuff(\n        $test: Input = { key1: [\"value\", \"value2\"], key2: { key3: 4 } }\n      ) {\n        allStuff(test: $test) {\n          people {\n            name\n          }\n        }\n      }\n    "], gql(_a));
        var result = {
            allStuff: {
                people: [
                    {
                        name: 'Luke Skywalker',
                    },
                    {
                        name: 'Jabba The Hutt',
                    },
                ],
            },
        };
        var variables = {
            test: { key1: ['value', 'value2'], key2: { key3: 4 } },
        };
        var link = mockSingleLink({
            request: { query: query, variables: variables },
            result: { data: result },
        });
        var client = new ApolloClient({
            link: link,
            cache: new InMemoryCache({}, { addTypename: false }),
            addTypename: false,
        });
        var basic = client.query({ query: query, variables: variables }).then(function (actualResult) {
            assert.deepEqual(actualResult.data, result);
        });
        var withDefault = client.query({ query: query }).then(function (actualResult) {
            assert.deepEqual(actualResult.data, result);
        });
        return Promise.all([basic, withDefault]);
        var _a;
    });
    it('should allow for a single query with default values that get overridden with variables', function () {
        var query = (_a = ["\n      query people($first: Int = 1) {\n        allPeople(first: $first) {\n          people {\n            name\n          }\n        }\n      }\n    "], _a.raw = ["\n      query people($first: Int = 1) {\n        allPeople(first: $first) {\n          people {\n            name\n          }\n        }\n      }\n    "], gql(_a));
        var variables = { first: 1 };
        var override = { first: 2 };
        var result = {
            allPeople: {
                people: [
                    {
                        name: 'Luke Skywalker',
                    },
                ],
            },
        };
        var overriddenResult = {
            allPeople: {
                people: [
                    {
                        name: 'Luke Skywalker',
                    },
                    {
                        name: 'Jabba The Hutt',
                    },
                ],
            },
        };
        var link = mockSingleLink({
            request: { query: query, variables: variables },
            result: { data: result },
        }, {
            request: { query: query, variables: override },
            result: { data: overriddenResult },
        });
        var client = new ApolloClient({
            link: link,
            cache: new InMemoryCache({}, { addTypename: false }),
            addTypename: false,
        });
        var basic = client.query({ query: query, variables: variables }).then(function (actualResult) {
            assert.deepEqual(actualResult.data, result);
        });
        var withDefault = client.query({ query: query }).then(function (actualResult) {
            return assert.deepEqual(actualResult.data, result);
        });
        var withOverride = client
            .query({ query: query, variables: override })
            .then(function (actualResult) {
            return assert.deepEqual(actualResult.data, overriddenResult);
        });
        return Promise.all([basic, withDefault, withOverride]);
        var _a;
    });
    it('should allow fragments on root query', function () {
        var query = (_a = ["\n      query {\n        records {\n          id\n          __typename\n        }\n        ...QueryFragment\n      }\n\n      fragment QueryFragment on Query {\n        records {\n          name\n          __typename\n        }\n        __typename\n      }\n    "], _a.raw = ["\n      query {\n        records {\n          id\n          __typename\n        }\n        ...QueryFragment\n      }\n\n      fragment QueryFragment on Query {\n        records {\n          name\n          __typename\n        }\n        __typename\n      }\n    "], gql(_a));
        var data = {
            records: [
                { id: 1, name: 'One', __typename: 'Record' },
                { id: 2, name: 'Two', __typename: 'Record' },
            ],
            __typename: 'Query',
        };
        var ifm = new IntrospectionFragmentMatcher({
            introspectionQueryResultData: {
                __schema: {
                    types: [
                        {
                            kind: 'UNION',
                            name: 'Query',
                            possibleTypes: [
                                {
                                    name: 'Record',
                                },
                            ],
                        },
                    ],
                },
            },
        });
        return clientRoundtrip(query, { data: data }, null, ifm);
        var _a;
    });
    it('store can be rehydrated from the server', function () {
        var query = (_a = ["\n      query people {\n        allPeople(first: 1) {\n          people {\n            name\n          }\n        }\n      }\n    "], _a.raw = ["\n      query people {\n        allPeople(first: 1) {\n          people {\n            name\n          }\n        }\n      }\n    "], gql(_a));
        var data = {
            allPeople: {
                people: [
                    {
                        name: 'Luke Skywalker',
                    },
                ],
            },
        };
        var link = mockSingleLink({
            request: { query: query },
            result: { data: data },
        });
        var initialState = {
            data: {
                'ROOT_QUERY.allPeople({"first":"1"}).people.0': {
                    name: 'Luke Skywalker',
                },
                'ROOT_QUERY.allPeople({"first":1})': {
                    people: [
                        {
                            type: 'id',
                            generated: true,
                            id: 'ROOT_QUERY.allPeople({"first":"1"}).people.0',
                        },
                    ],
                },
                ROOT_QUERY: {
                    'allPeople({"first":1})': {
                        type: 'id',
                        id: 'ROOT_QUERY.allPeople({"first":1})',
                        generated: true,
                    },
                },
                optimistic: [],
            },
        };
        var finalState = assign({}, initialState, {});
        var client = new ApolloClient({
            link: link,
            cache: new InMemoryCache(initialState.data, { addTypename: false }),
            addTypename: false,
        });
        return client.query({ query: query }).then(function (result) {
            assert.deepEqual(result.data, data);
            assert.deepEqual(finalState.data, client.queryManager.dataStore.getCache().getData());
        });
        var _a;
    });
    it('should return errors correctly for a single query', function () {
        var query = (_a = ["\n      query people {\n        allPeople(first: 1) {\n          people {\n            name\n          }\n        }\n      }\n    "], _a.raw = ["\n      query people {\n        allPeople(first: 1) {\n          people {\n            name\n          }\n        }\n      }\n    "], gql(_a));
        var errors = [
            {
                name: 'test',
                message: 'Syntax Error GraphQL request (8:9) Expected Name, found EOF',
            },
        ];
        var link = mockSingleLink({
            request: { query: query },
            result: { errors: errors },
        });
        var client = new ApolloClient({
            link: link,
            cache: new InMemoryCache({}, { addTypename: false }),
            addTypename: false,
        });
        return client.query({ query: query }).catch(function (error) {
            assert.deepEqual(error.graphQLErrors, errors);
        });
        var _a;
    });
    it('should return GraphQL errors correctly for a single query with an apollo-link enabled network interface', function (done) {
        var query = (_a = ["\n      query people {\n        allPeople(first: 1) {\n          people {\n            name\n          }\n        }\n      }\n    "], _a.raw = ["\n      query people {\n        allPeople(first: 1) {\n          people {\n            name\n          }\n        }\n      }\n    "], gql(_a));
        var data = {
            allPeople: {
                people: [
                    {
                        name: 'Luke Skywalker',
                    },
                ],
            },
        };
        var errors = [
            {
                name: 'test',
                message: 'Syntax Error GraphQL request (8:9) Expected Name, found EOF',
            },
        ];
        var link = ApolloLink.from([
            function () {
                return new Observable(function (observer) {
                    observer.next({ data: data, errors: errors });
                });
            },
        ]);
        var client = new ApolloClient({
            link: link,
            cache: new InMemoryCache({}, { addTypename: false }),
            addTypename: false,
        });
        client.query({ query: query }).catch(function (error) {
            assert.deepEqual(error.graphQLErrors, errors);
            done();
        });
        var _a;
    });
    it('should pass a network error correctly on a query using an observable network interface with a warning', function (done) {
        withWarning(function () {
            var query = (_a = ["\n        query people {\n          allPeople(first: 1) {\n            people {\n              name\n            }\n          }\n        }\n      "], _a.raw = ["\n        query people {\n          allPeople(first: 1) {\n            people {\n              name\n            }\n          }\n        }\n      "], gql(_a));
            var networkError = new Error('Some kind of network error.');
            var link = ApolloLink.from([
                function () {
                    return new Observable(function (_) {
                        throw networkError;
                    });
                },
            ]);
            var client = new ApolloClient({
                link: link,
                cache: new InMemoryCache({}, { addTypename: false }),
                addTypename: false,
            });
            client.query({ query: query }).catch(function (error) {
                assert(error.networkError);
                assert.deepEqual(error.networkError.message, networkError.message);
                done();
            });
            var _a;
        }, /deprecated/);
    });
    it('should pass a network error correctly on a query with apollo-link network interface', function (done) {
        var query = (_a = ["\n      query people {\n        allPeople(first: 1) {\n          people {\n            name\n          }\n        }\n      }\n    "], _a.raw = ["\n      query people {\n        allPeople(first: 1) {\n          people {\n            name\n          }\n        }\n      }\n    "], gql(_a));
        var networkError = new Error('Some kind of network error.');
        var link = ApolloLink.from([
            function () {
                return new Observable(function (_) {
                    throw networkError;
                });
            },
        ]);
        var client = new ApolloClient({
            link: link,
            cache: new InMemoryCache({}, { addTypename: false }),
            addTypename: false,
        });
        client.query({ query: query }).catch(function (error) {
            assert(error.networkError);
            assert.deepEqual(error.networkError.message, networkError.message);
            done();
        });
        var _a;
    });
    it('should warn when receiving multiple results from apollo-link network interface', function () {
        var query = (_a = ["\n      query people {\n        allPeople(first: 1) {\n          people {\n            name\n          }\n        }\n      }\n    "], _a.raw = ["\n      query people {\n        allPeople(first: 1) {\n          people {\n            name\n          }\n        }\n      }\n    "], gql(_a));
        var data = {
            allPeople: {
                people: [
                    {
                        name: 'Luke Skywalker',
                    },
                ],
            },
        };
        var link = ApolloLink.from([function () { return Observable.of({ data: data }, { data: data }); }]);
        var client = new ApolloClient({
            link: link,
            cache: new InMemoryCache({}, { addTypename: false }),
            addTypename: false,
        });
        return withWarning(function () {
            return client.query({ query: query }).then(function (result) {
                assert.deepEqual(result.data, data);
            });
        }, /multiple results/);
        var _a;
    });
    it('should surface errors in observer.next as uncaught', function (done) {
        var expectedError = new Error('this error should not reach the store');
        var listeners = process.listeners('uncaughtException');
        var oldHandler = listeners[listeners.length - 1];
        var handleUncaught = function (e) {
            process.removeListener('uncaughtException', handleUncaught);
            process.addListener('uncaughtException', oldHandler);
            if (e === expectedError) {
                done();
            }
            else {
                done(e);
            }
        };
        process.removeListener('uncaughtException', oldHandler);
        process.addListener('uncaughtException', handleUncaught);
        var query = (_a = ["\n      query people {\n        allPeople(first: 1) {\n          people {\n            name\n          }\n        }\n      }\n    "], _a.raw = ["\n      query people {\n        allPeople(first: 1) {\n          people {\n            name\n          }\n        }\n      }\n    "], gql(_a));
        var data = {
            allPeople: {
                people: [
                    {
                        name: 'Luke Skywalker',
                    },
                ],
            },
        };
        var link = mockSingleLink({
            request: { query: query },
            result: { data: data },
        });
        var client = new ApolloClient({
            link: link,
            cache: new InMemoryCache({}, { addTypename: false }),
            addTypename: false,
        });
        var handle = client.watchQuery({ query: query });
        handle.subscribe({
            next: function () {
                throw expectedError;
            },
        });
        var _a;
    });
    it('should surfaces errors in observer.error as uncaught', function (done) {
        var expectedError = new Error('this error should not reach the store');
        var listeners = process.listeners('uncaughtException');
        var oldHandler = listeners[listeners.length - 1];
        var handleUncaught = function (e) {
            process.removeListener('uncaughtException', handleUncaught);
            process.addListener('uncaughtException', oldHandler);
            if (e === expectedError) {
                done();
            }
            else {
                done(e);
            }
        };
        process.removeListener('uncaughtException', oldHandler);
        process.addListener('uncaughtException', handleUncaught);
        var query = (_a = ["\n      query people {\n        allPeople(first: 1) {\n          people {\n            name\n          }\n        }\n      }\n    "], _a.raw = ["\n      query people {\n        allPeople(first: 1) {\n          people {\n            name\n          }\n        }\n      }\n    "], gql(_a));
        var link = mockSingleLink({
            request: { query: query },
            result: {},
        });
        var client = new ApolloClient({
            link: link,
            cache: new InMemoryCache({}, { addTypename: false }),
            addTypename: false,
        });
        var handle = client.watchQuery({ query: query });
        handle.subscribe({
            next: function () {
                done(new Error('did not expect next to be called'));
            },
            error: function () {
                throw expectedError;
            },
        });
        var _a;
    });
    it('should allow for subscribing to a request', function (done) {
        var query = (_a = ["\n      query people {\n        allPeople(first: 1) {\n          people {\n            name\n          }\n        }\n      }\n    "], _a.raw = ["\n      query people {\n        allPeople(first: 1) {\n          people {\n            name\n          }\n        }\n      }\n    "], gql(_a));
        var data = {
            allPeople: {
                people: [
                    {
                        name: 'Luke Skywalker',
                    },
                ],
            },
        };
        var link = mockSingleLink({
            request: { query: query },
            result: { data: data },
        });
        var client = new ApolloClient({
            link: link,
            cache: new InMemoryCache({}, { addTypename: false }),
            addTypename: false,
        });
        var handle = client.watchQuery({ query: query });
        handle.subscribe({
            next: function (result) {
                assert.deepEqual(result.data, data);
                done();
            },
        });
        var _a;
    });
    it('should be able to transform queries', function () {
        var query = (_a = ["\n      query {\n        author {\n          firstName\n          lastName\n        }\n      }\n    "], _a.raw = ["\n      query {\n        author {\n          firstName\n          lastName\n        }\n      }\n    "], gql(_a));
        var transformedQuery = (_b = ["\n      query {\n        author {\n          firstName\n          lastName\n          __typename\n        }\n      }\n    "], _b.raw = ["\n      query {\n        author {\n          firstName\n          lastName\n          __typename\n        }\n      }\n    "], gql(_b));
        var result = {
            author: {
                firstName: 'John',
                lastName: 'Smith',
            },
        };
        var transformedResult = {
            author: {
                firstName: 'John',
                lastName: 'Smith',
                __typename: 'Author',
            },
        };
        var link = mockSingleLink({
            request: { query: query },
            result: { data: result },
        }, {
            request: { query: transformedQuery },
            result: { data: transformedResult },
        });
        var client = new ApolloClient({
            link: link,
            cache: new InMemoryCache(),
            addTypename: true,
        });
        return client.query({ query: query }).then(function (actualResult) {
            assert.deepEqual(actualResult.data, transformedResult);
        });
        var _a, _b;
    });
    it('should be able to transform queries on network-only fetches', function () {
        var query = (_a = ["\n      query {\n        author {\n          firstName\n          lastName\n        }\n      }\n    "], _a.raw = ["\n      query {\n        author {\n          firstName\n          lastName\n        }\n      }\n    "], gql(_a));
        var transformedQuery = (_b = ["\n      query {\n        author {\n          firstName\n          lastName\n          __typename\n        }\n      }\n    "], _b.raw = ["\n      query {\n        author {\n          firstName\n          lastName\n          __typename\n        }\n      }\n    "], gql(_b));
        var result = {
            author: {
                firstName: 'John',
                lastName: 'Smith',
            },
        };
        var transformedResult = {
            author: {
                firstName: 'John',
                lastName: 'Smith',
                __typename: 'Author',
            },
        };
        var link = mockSingleLink({
            request: { query: query },
            result: { data: result },
        }, {
            request: { query: transformedQuery },
            result: { data: transformedResult },
        });
        var client = new ApolloClient({
            link: link,
            cache: new InMemoryCache(),
            addTypename: true,
        });
        return client
            .query({ fetchPolicy: 'network-only', query: query })
            .then(function (actualResult) {
            assert.deepEqual(actualResult.data, transformedResult);
        });
        var _a, _b;
    });
    it('should handle named fragments on mutations', function () {
        var mutation = (_a = ["\n      mutation {\n        starAuthor(id: 12) {\n          author {\n            __typename\n            ...authorDetails\n          }\n        }\n      }\n\n      fragment authorDetails on Author {\n        firstName\n        lastName\n      }\n    "], _a.raw = ["\n      mutation {\n        starAuthor(id: 12) {\n          author {\n            __typename\n            ...authorDetails\n          }\n        }\n      }\n\n      fragment authorDetails on Author {\n        firstName\n        lastName\n      }\n    "], gql(_a));
        var result = {
            starAuthor: {
                author: {
                    __typename: 'Author',
                    firstName: 'John',
                    lastName: 'Smith',
                },
            },
        };
        var link = mockSingleLink({
            request: { query: mutation },
            result: { data: result },
        });
        var client = new ApolloClient({
            link: link,
            cache: new InMemoryCache({}, { addTypename: false }),
            addTypename: false,
        });
        return client.mutate({ mutation: mutation }).then(function (actualResult) {
            assert.deepEqual(actualResult.data, result);
        });
        var _a;
    });
    it('should be able to handle named fragments on network-only queries', function () {
        var query = (_a = ["\n      fragment authorDetails on Author {\n        firstName\n        lastName\n      }\n\n      query {\n        author {\n          __typename\n          ...authorDetails\n        }\n      }\n    "], _a.raw = ["\n      fragment authorDetails on Author {\n        firstName\n        lastName\n      }\n\n      query {\n        author {\n          __typename\n          ...authorDetails\n        }\n      }\n    "], gql(_a));
        var result = {
            author: {
                __typename: 'Author',
                firstName: 'John',
                lastName: 'Smith',
            },
        };
        var link = mockSingleLink({
            request: { query: query },
            result: { data: result },
        });
        var client = new ApolloClient({
            link: link,
            cache: new InMemoryCache({}, { addTypename: false }),
            addTypename: false,
        });
        return client
            .query({ fetchPolicy: 'network-only', query: query })
            .then(function (actualResult) {
            assert.deepEqual(actualResult.data, result);
        });
        var _a;
    });
    it('should be able to handle named fragments with multiple fragments', function () {
        var query = (_a = ["\n      query {\n        author {\n          __typename\n          ...authorDetails\n          ...moreDetails\n        }\n      }\n\n      fragment authorDetails on Author {\n        firstName\n        lastName\n      }\n\n      fragment moreDetails on Author {\n        address\n      }\n    "], _a.raw = ["\n      query {\n        author {\n          __typename\n          ...authorDetails\n          ...moreDetails\n        }\n      }\n\n      fragment authorDetails on Author {\n        firstName\n        lastName\n      }\n\n      fragment moreDetails on Author {\n        address\n      }\n    "], gql(_a));
        var result = {
            author: {
                __typename: 'Author',
                firstName: 'John',
                lastName: 'Smith',
                address: '1337 10th St.',
            },
        };
        var link = mockSingleLink({
            request: { query: query },
            result: { data: result },
        });
        var client = new ApolloClient({
            link: link,
            cache: new InMemoryCache({}, { addTypename: false }),
            addTypename: false,
        });
        return client.query({ query: query }).then(function (actualResult) {
            assert.deepEqual(actualResult.data, result);
        });
        var _a;
    });
    it('should be able to handle named fragments', function () {
        var query = (_a = ["\n      query {\n        author {\n          __typename\n          ...authorDetails\n        }\n      }\n\n      fragment authorDetails on Author {\n        firstName\n        lastName\n      }\n    "], _a.raw = ["\n      query {\n        author {\n          __typename\n          ...authorDetails\n        }\n      }\n\n      fragment authorDetails on Author {\n        firstName\n        lastName\n      }\n    "], gql(_a));
        var result = {
            author: {
                __typename: 'Author',
                firstName: 'John',
                lastName: 'Smith',
            },
        };
        var link = mockSingleLink({
            request: { query: query },
            result: { data: result },
        });
        var client = new ApolloClient({
            link: link,
            cache: new InMemoryCache({}, { addTypename: false }),
            addTypename: false,
        });
        return client.query({ query: query }).then(function (actualResult) {
            assert.deepEqual(actualResult.data, result);
        });
        var _a;
    });
    it('should be able to handle inlined fragments on an Interface type', function () {
        var query = (_a = ["\n      query items {\n        items {\n          ...ItemFragment\n          __typename\n        }\n      }\n\n      fragment ItemFragment on Item {\n        id\n        __typename\n        ... on ColorItem {\n          color\n          __typename\n        }\n      }\n    "], _a.raw = ["\n      query items {\n        items {\n          ...ItemFragment\n          __typename\n        }\n      }\n\n      fragment ItemFragment on Item {\n        id\n        __typename\n        ... on ColorItem {\n          color\n          __typename\n        }\n      }\n    "], gql(_a));
        var result = {
            items: [
                {
                    __typename: 'ColorItem',
                    id: '27tlpoPeXm6odAxj3paGQP',
                    color: 'red',
                },
                {
                    __typename: 'MonochromeItem',
                    id: '1t3iFLsHBm4c4RjOMdMgOO',
                },
            ],
        };
        var fancyFragmentMatcher = function (idValue, typeCondition, context) {
            var obj = context.store[idValue.id];
            if (!obj) {
                return false;
            }
            var implementingTypesMap = {
                Item: ['ColorItem', 'MonochromeItem'],
            };
            if (obj.__typename === typeCondition) {
                return true;
            }
            var implementingTypes = implementingTypesMap[typeCondition];
            if (implementingTypes && implementingTypes.indexOf(obj.__typename) > -1) {
                return true;
            }
            return false;
        };
        var link = mockSingleLink({
            request: { query: query },
            result: { data: result },
        });
        var client = new ApolloClient({
            link: link,
            cache: new InMemoryCache({}, {
                fragmentMatcher: fancyFragmentMatcher,
            }),
        });
        return client.query({ query: query }).then(function (actualResult) {
            assert.deepEqual(actualResult.data, result);
        });
        var _a;
    });
    it('should be able to handle inlined fragments on an Interface type with introspection fragment matcher', function () {
        var query = (_a = ["\n      query items {\n        items {\n          ...ItemFragment\n          __typename\n        }\n      }\n\n      fragment ItemFragment on Item {\n        id\n        ... on ColorItem {\n          color\n          __typename\n        }\n        __typename\n      }\n    "], _a.raw = ["\n      query items {\n        items {\n          ...ItemFragment\n          __typename\n        }\n      }\n\n      fragment ItemFragment on Item {\n        id\n        ... on ColorItem {\n          color\n          __typename\n        }\n        __typename\n      }\n    "], gql(_a));
        var result = {
            items: [
                {
                    __typename: 'ColorItem',
                    id: '27tlpoPeXm6odAxj3paGQP',
                    color: 'red',
                },
                {
                    __typename: 'MonochromeItem',
                    id: '1t3iFLsHBm4c4RjOMdMgOO',
                },
            ],
        };
        var link = mockSingleLink({
            request: { query: query },
            result: { data: result },
        });
        var ifm = new IntrospectionFragmentMatcher({
            introspectionQueryResultData: {
                __schema: {
                    types: [
                        {
                            kind: 'UNION',
                            name: 'Item',
                            possibleTypes: [
                                {
                                    name: 'ColorItem',
                                },
                                {
                                    name: 'MonochromeItem',
                                },
                            ],
                        },
                    ],
                },
            },
        });
        var client = new ApolloClient({
            link: link,
            cache: new InMemoryCache({}, { fragmentMatcher: ifm.match }),
        });
        return client.query({ query: query }).then(function (actualResult) {
            assert.deepEqual(actualResult.data, result);
        });
        var _a;
    });
    it('should call updateQueries and update after mutation on query with inlined fragments on an Interface type', function (done) {
        var query = (_a = ["\n      query items {\n        items {\n          ...ItemFragment\n          __typename\n        }\n      }\n\n      fragment ItemFragment on Item {\n        id\n        ... on ColorItem {\n          color\n          __typename\n        }\n        __typename\n      }\n    "], _a.raw = ["\n      query items {\n        items {\n          ...ItemFragment\n          __typename\n        }\n      }\n\n      fragment ItemFragment on Item {\n        id\n        ... on ColorItem {\n          color\n          __typename\n        }\n        __typename\n      }\n    "], gql(_a));
        var result = {
            items: [
                {
                    __typename: 'ColorItem',
                    id: '27tlpoPeXm6odAxj3paGQP',
                    color: 'red',
                },
                {
                    __typename: 'MonochromeItem',
                    id: '1t3iFLsHBm4c4RjOMdMgOO',
                },
            ],
        };
        var mutation = (_b = ["\n      mutation myMutationName {\n        fortuneCookie\n      }\n    "], _b.raw = ["\n      mutation myMutationName {\n        fortuneCookie\n      }\n    "], gql(_b));
        var mutationResult = {
            fortuneCookie: 'The waiter spit in your food',
        };
        var link = mockSingleLink({
            request: { query: query },
            result: { data: result },
        }, {
            request: { query: mutation },
            result: { data: mutationResult },
        });
        var ifm = new IntrospectionFragmentMatcher({
            introspectionQueryResultData: {
                __schema: {
                    types: [
                        {
                            kind: 'UNION',
                            name: 'Item',
                            possibleTypes: [
                                {
                                    name: 'ColorItem',
                                },
                                {
                                    name: 'MonochromeItem',
                                },
                            ],
                        },
                    ],
                },
            },
        });
        var client = new ApolloClient({
            link: link,
            cache: new InMemoryCache({}, { fragmentMatcher: ifm.match }),
        });
        var queryUpdaterSpy = sinon.spy();
        var queryUpdater = function (prev) {
            queryUpdaterSpy();
            return prev;
        };
        var updateQueries = {
            items: queryUpdater,
        };
        var updateSpy = sinon.spy();
        var obs = client.watchQuery({ query: query });
        var sub = obs.subscribe({
            next: function () {
                client
                    .mutate({ mutation: mutation, updateQueries: updateQueries, update: updateSpy })
                    .then(function () {
                    assert.isTrue(queryUpdaterSpy.called);
                    assert.isTrue(updateSpy.called);
                    sub.unsubscribe();
                    done();
                })
                    .catch(function (err) {
                    done(err);
                });
            },
            error: function (err) {
                done(err);
            },
        });
        var _a, _b;
    });
    it('should send operationName along with the query to the server', function () {
        var query = (_a = ["\n      query myQueryName {\n        fortuneCookie\n      }\n    "], _a.raw = ["\n      query myQueryName {\n        fortuneCookie\n      }\n    "], gql(_a));
        var data = {
            fortuneCookie: 'The waiter spit in your food',
        };
        var link = ApolloLink.from([
            function (request) {
                assert.equal(request.operationName, 'myQueryName');
                return Observable.of({ data: data });
            },
        ]);
        var client = new ApolloClient({
            link: link,
            cache: new InMemoryCache({}, { addTypename: false }),
            addTypename: false,
        });
        return client.query({ query: query }).then(function (actualResult) {
            assert.deepEqual(actualResult.data, data);
        });
        var _a;
    });
    it('should send operationName along with the mutation to the server', function () {
        var mutation = (_a = ["\n      mutation myMutationName {\n        fortuneCookie\n      }\n    "], _a.raw = ["\n      mutation myMutationName {\n        fortuneCookie\n      }\n    "], gql(_a));
        var data = {
            fortuneCookie: 'The waiter spit in your food',
        };
        var link = ApolloLink.from([
            function (request) {
                assert.equal(request.operationName, 'myMutationName');
                return Observable.of({ data: data });
            },
        ]);
        var client = new ApolloClient({
            link: link,
            cache: new InMemoryCache({}, { addTypename: false }),
            addTypename: false,
        });
        return client.mutate({ mutation: mutation }).then(function (actualResult) {
            assert.deepEqual(actualResult.data, data);
        });
        var _a;
    });
    it('does not deduplicate queries if option is set to false', function () {
        var queryDoc = (_a = ["\n      query {\n        author {\n          name\n        }\n      }\n    "], _a.raw = ["\n      query {\n        author {\n          name\n        }\n      }\n    "], gql(_a));
        var data = {
            author: {
                name: 'Jonas',
            },
        };
        var data2 = {
            author: {
                name: 'Dhaivat',
            },
        };
        var link = mockSingleLink({
            request: { query: queryDoc },
            result: { data: data },
            delay: 10,
        }, {
            request: { query: queryDoc },
            result: { data: data2 },
        });
        var client = new ApolloClient({
            link: link,
            cache: new InMemoryCache({}, { addTypename: false }),
            addTypename: false,
            queryDeduplication: false,
        });
        var q1 = client.query({ query: queryDoc });
        var q2 = client.query({ query: queryDoc });
        return Promise.all([q1, q2]).then(function (_a) {
            var result1 = _a[0], result2 = _a[1];
            assert.deepEqual(result1.data, data);
            assert.deepEqual(result2.data, data2);
        });
        var _a;
    });
    it('deduplicates queries by default', function () {
        var queryDoc = (_a = ["\n      query {\n        author {\n          name\n        }\n      }\n    "], _a.raw = ["\n      query {\n        author {\n          name\n        }\n      }\n    "], gql(_a));
        var data = {
            author: {
                name: 'Jonas',
            },
        };
        var data2 = {
            author: {
                name: 'Dhaivat',
            },
        };
        var link = mockSingleLink({
            request: { query: queryDoc },
            result: { data: data },
            delay: 10,
        }, {
            request: { query: queryDoc },
            result: { data: data2 },
        });
        var client = new ApolloClient({
            link: link,
            cache: new InMemoryCache({}, { addTypename: false }),
            addTypename: false,
        });
        var q1 = client.query({ query: queryDoc });
        var q2 = client.query({ query: queryDoc });
        return Promise.all([q1, q2]).then(function (_a) {
            var result1 = _a[0], result2 = _a[1];
            assert.deepEqual(result1.data, result2.data);
        });
        var _a;
    });
    describe('deprecated options', function () {
        var query = (_a = ["\n      query people {\n        name\n      }\n    "], _a.raw = ["\n      query people {\n        name\n      }\n    "], gql(_a));
        it('errors when returnPartialData is used on query', function () {
            var client = new ApolloClient({
                link: ApolloLink.empty(),
                cache: new InMemoryCache(),
            });
            assert.throws(function () {
                client.query({ query: query, returnPartialData: true });
            }, /returnPartialData/);
        });
        it('errors when returnPartialData is used on watchQuery', function () {
            var client = new ApolloClient({
                link: ApolloLink.empty(),
                cache: new InMemoryCache(),
            });
            assert.throws(function () {
                client.query({ query: query, returnPartialData: true });
            }, /returnPartialData/);
        });
        var _a;
    });
    describe('accepts dataIdFromObject option', function () {
        var query = (_a = ["\n      query people {\n        allPeople(first: 1) {\n          people {\n            id\n            name\n          }\n        }\n      }\n    "], _a.raw = ["\n      query people {\n        allPeople(first: 1) {\n          people {\n            id\n            name\n          }\n        }\n      }\n    "], gql(_a));
        var data = {
            allPeople: {
                people: [
                    {
                        id: '1',
                        name: 'Luke Skywalker',
                    },
                ],
            },
        };
        it('for internal store', function () {
            var link = mockSingleLink({
                request: { query: query },
                result: { data: data },
            });
            var client = new ApolloClient({
                link: link,
                cache: new InMemoryCache({}, {
                    addTypename: false,
                    dataIdFromObject: function (obj) { return obj.id; },
                }),
                addTypename: false,
            });
            return client.query({ query: query }).then(function (result) {
                assert.deepEqual(result.data, data);
                assert.deepEqual(client.queryManager.dataStore.getCache().getData()['1'], {
                    id: '1',
                    name: 'Luke Skywalker',
                });
            });
        });
        var _a;
    });
    describe('cache-and-network fetchPolicy', function () {
        var query = (_a = ["\n      query number {\n        myNumber {\n          n\n        }\n      }\n    "], _a.raw = ["\n      query number {\n        myNumber {\n          n\n        }\n      }\n    "], gql(_a));
        var initialData = {
            myNumber: {
                n: 1,
            },
        };
        var networkFetch = {
            myNumber: {
                n: 2,
            },
        };
        it('errors when being used on query', function () {
            var client = new ApolloClient({
                link: ApolloLink.empty(),
                cache: new InMemoryCache(),
            });
            assert.throws(function () {
                client.query({ query: query, fetchPolicy: 'cache-and-network' });
            });
        });
        it('fetches from cache first, then network', function (done) {
            var link = mockSingleLink({
                request: { query: query },
                result: { data: networkFetch },
            });
            var client = new ApolloClient({
                link: link,
                cache: new InMemoryCache({}, { addTypename: false }),
                addTypename: false,
            });
            client.writeQuery({
                query: query,
                data: initialData,
            });
            var obs = client.watchQuery({
                query: query,
                fetchPolicy: 'cache-and-network',
            });
            subscribeAndCount(done, obs, function (handleCount, result) {
                if (handleCount === 1) {
                    assert.deepEqual(result.data, initialData);
                }
                else if (handleCount === 2) {
                    assert.deepEqual(result.data, networkFetch);
                    done();
                }
            });
        });
        it('does not fail if cache entry is not present', function (done) {
            var link = mockSingleLink({
                request: { query: query },
                result: { data: networkFetch },
            });
            var client = new ApolloClient({
                link: link,
                cache: new InMemoryCache({}, { addTypename: false }),
                addTypename: false,
            });
            var obs = client.watchQuery({
                query: query,
                fetchPolicy: 'cache-and-network',
            });
            subscribeAndCount(done, obs, function (handleCount, result) {
                if (handleCount === 1) {
                    assert.equal(result.data, undefined);
                    assert(result.loading);
                }
                else if (handleCount === 2) {
                    assert.deepEqual(result.data, networkFetch);
                    assert(!result.loading);
                    done();
                }
            });
        });
        it('fails if network request fails', function (done) {
            var link = mockSingleLink();
            var client = new ApolloClient({
                link: link,
                cache: new InMemoryCache({}, { addTypename: false }),
                addTypename: false,
            });
            var obs = client.watchQuery({
                query: query,
                fetchPolicy: 'cache-and-network',
            });
            var count = 0;
            obs.subscribe({
                next: function (result) {
                    assert.equal(result.data, undefined);
                    assert(result.loading);
                    count++;
                },
                error: function (e) {
                    assert.match(e.message, /No more mocked responses/);
                    assert.equal(count, 1);
                    done();
                },
            });
        });
        var _a;
    });
    describe('standby queries', function () {
        it('cannot be started with watchQuery or query', function () {
            var client = new ApolloClient({
                link: ApolloLink.empty(),
                cache: new InMemoryCache(),
            });
            assert.throws(function () {
                return client.watchQuery({
                    query: (_a = ["\n              {\n                abc\n              }\n            "], _a.raw = ["\n              {\n                abc\n              }\n            "], gql(_a)),
                    fetchPolicy: 'standby',
                });
                var _a;
            }, 'client.watchQuery cannot be called with fetchPolicy set to "standby"');
        });
        it('are not watching the store or notifying on updates', function (done) {
            var query = (_a = ["\n        {\n          test\n        }\n      "], _a.raw = ["\n        {\n          test\n        }\n      "], gql(_a));
            var data = { test: 'ok' };
            var data2 = { test: 'not ok' };
            var link = mockSingleLink({
                request: { query: query },
                result: { data: data },
            });
            var client = new ApolloClient({ link: link, cache: new InMemoryCache() });
            var obs = client.watchQuery({ query: query, fetchPolicy: 'cache-first' });
            var handleCalled = false;
            subscribeAndCount(done, obs, function (handleCount, result) {
                if (handleCount === 1) {
                    assert.deepEqual(result.data, data);
                    obs.setOptions({ fetchPolicy: 'standby' }).then(function () {
                        client.writeQuery({ query: query, data: data2 });
                    });
                    setTimeout(function () {
                        if (!handleCalled) {
                            done();
                        }
                    }, 20);
                }
                if (handleCount === 2) {
                    handleCalled = true;
                    done(new Error('Handle should never be called on standby query'));
                }
            });
            var _a;
        });
        it('return the current result when coming out of standby', function (done) {
            var query = (_a = ["\n        {\n          test\n        }\n      "], _a.raw = ["\n        {\n          test\n        }\n      "], gql(_a));
            var data = { test: 'ok' };
            var data2 = { test: 'not ok' };
            var link = mockSingleLink({
                request: { query: query },
                result: { data: data },
            });
            var client = new ApolloClient({ link: link, cache: new InMemoryCache() });
            var obs = client.watchQuery({ query: query, fetchPolicy: 'cache-first' });
            var handleCalled = false;
            subscribeAndCount(done, obs, function (handleCount, result) {
                if (handleCount === 1) {
                    assert.deepEqual(result.data, data);
                    obs.setOptions({ fetchPolicy: 'standby' }).then(function () {
                        client.writeQuery({ query: query, data: data2 });
                        setTimeout(function () {
                            obs.setOptions({ fetchPolicy: 'cache-first' });
                        }, 10);
                    });
                }
                if (handleCount === 2) {
                    handleCalled = true;
                    assert.deepEqual(result.data, data2);
                    done();
                }
            });
            var _a;
        });
    });
    describe('network-only fetchPolicy', function () {
        var query = (_a = ["\n      query number {\n        myNumber {\n          n\n        }\n      }\n    "], _a.raw = ["\n      query number {\n        myNumber {\n          n\n        }\n      }\n    "], gql(_a));
        var firstFetch = {
            myNumber: {
                n: 1,
            },
        };
        var secondFetch = {
            myNumber: {
                n: 2,
            },
        };
        var link;
        var clock;
        beforeEach(function () {
            link = mockSingleLink({
                request: { query: query },
                result: { data: firstFetch },
            }, {
                request: { query: query },
                result: { data: secondFetch },
            });
        });
        afterEach(function () {
            if (clock) {
                clock.restore();
            }
        });
        it('forces the query to rerun', function () {
            var client = new ApolloClient({
                link: link,
                cache: new InMemoryCache({}, { addTypename: false }),
                addTypename: false,
            });
            return (client
                .query({ query: query })
                .then(function () { return client.query({ query: query, fetchPolicy: 'network-only' }); })
                .then(function (result) {
                assert.deepEqual(result.data, { myNumber: { n: 2 } });
            }));
        });
        it('can be disabled with ssrMode', function () {
            var client = new ApolloClient({
                link: link,
                ssrMode: true,
                cache: new InMemoryCache({}, { addTypename: false }),
                addTypename: false,
            });
            var options = { query: query, fetchPolicy: 'network-only' };
            return (client
                .query({ query: query })
                .then(function () { return client.query(options); })
                .then(function (result) {
                assert.deepEqual(result.data, { myNumber: { n: 1 } });
                assert.deepEqual(options, {
                    query: query,
                    fetchPolicy: 'network-only',
                });
            }));
        });
        it('can temporarily be disabled with ssrForceFetchDelay', function () {
            clock = sinon.useFakeTimers();
            var client = new ApolloClient({
                link: link,
                ssrForceFetchDelay: 100,
                cache: new InMemoryCache({}, { addTypename: false }),
                addTypename: false,
            });
            var outerPromise = client
                .query({ query: query })
                .then(function () {
                var promise = client.query({ query: query, fetchPolicy: 'network-only' });
                clock.tick(0);
                return promise;
            })
                .then(function (result) {
                assert.deepEqual(result.data, { myNumber: { n: 1 } });
                clock.tick(100);
                var promise = client.query({ query: query, fetchPolicy: 'network-only' });
                clock.tick(0);
                return promise;
            })
                .then(function (result) {
                assert.deepEqual(result.data, { myNumber: { n: 2 } });
            });
            clock.tick(0);
            return outerPromise;
        });
        var _a;
    });
    it('should expose a method called printAST that is prints graphql queries', function () {
        var query = (_a = ["\n      query {\n        fortuneCookie\n      }\n    "], _a.raw = ["\n      query {\n        fortuneCookie\n      }\n    "], gql(_a));
        assert.equal(printAST(query), print(query));
        var _a;
    });
    it('should pass a network error correctly on a mutation', function (done) {
        var mutation = (_a = ["\n      mutation {\n        person {\n          firstName\n          lastName\n        }\n      }\n    "], _a.raw = ["\n      mutation {\n        person {\n          firstName\n          lastName\n        }\n      }\n    "], gql(_a));
        var data = {
            person: {
                firstName: 'John',
                lastName: 'Smith',
            },
        };
        var networkError = new Error('Some kind of network error.');
        var client = new ApolloClient({
            link: mockSingleLink({
                request: { query: mutation },
                result: { data: data },
                error: networkError,
            }),
            cache: new InMemoryCache({}, { addTypename: false }),
            addTypename: false,
        });
        client
            .mutate({ mutation: mutation })
            .then(function (_) {
            done(new Error('Returned a result when it should not have.'));
        })
            .catch(function (error) {
            assert(error.networkError);
            assert.equal(error.networkError.message, networkError.message);
            done();
        });
        var _a;
    });
    it('should pass a GraphQL error correctly on a mutation', function (done) {
        var mutation = (_a = ["\n      mutation {\n        newPerson {\n          person {\n            firstName\n            lastName\n          }\n        }\n      }\n    "], _a.raw = ["\n      mutation {\n        newPerson {\n          person {\n            firstName\n            lastName\n          }\n        }\n      }\n    "], gql(_a));
        var data = {
            person: {
                firstName: 'John',
                lastName: 'Smith',
            },
        };
        var errors = [new Error('Some kind of GraphQL error.')];
        var client = new ApolloClient({
            link: mockSingleLink({
                request: { query: mutation },
                result: { data: data, errors: errors },
            }),
            cache: new InMemoryCache({}, { addTypename: false }),
            addTypename: false,
        });
        client
            .mutate({ mutation: mutation })
            .then(function (_) {
            done(new Error('Returned a result when it should not have.'));
        })
            .catch(function (error) {
            assert(error.graphQLErrors);
            assert.equal(error.graphQLErrors.length, 1);
            assert.equal(error.graphQLErrors[0].message, errors[0].message);
            done();
        });
        var _a;
    });
    it('should rollback optimistic after mutation got a GraphQL error', function (done) {
        var mutation = (_a = ["\n      mutation {\n        newPerson {\n          person {\n            firstName\n            lastName\n          }\n        }\n      }\n    "], _a.raw = ["\n      mutation {\n        newPerson {\n          person {\n            firstName\n            lastName\n          }\n        }\n      }\n    "], gql(_a));
        var data = {
            newPerson: {
                person: {
                    firstName: 'John',
                    lastName: 'Smith',
                },
            },
        };
        var errors = [new Error('Some kind of GraphQL error.')];
        var client = new ApolloClient({
            link: mockSingleLink({
                request: { query: mutation },
                result: { data: data, errors: errors },
            }),
            cache: new InMemoryCache({}, { addTypename: false }),
            addTypename: false,
        });
        var mutatePromise = client.mutate({
            mutation: mutation,
            optimisticResponse: {
                newPerson: {
                    person: {
                        firstName: 'John*',
                        lastName: 'Smith*',
                    },
                },
            },
        });
        assert.equal(client.queryManager.dataStore.getCache().optimistic.length, 1);
        mutatePromise
            .then(function (_) {
            done(new Error('Returned a result when it should not have.'));
        })
            .catch(function (_) {
            assert.equal(client.queryManager.dataStore.getCache().optimistic.length, 0);
            done();
        });
        var _a;
    });
    it('has a resetStore method which calls QueryManager', function (done) {
        var client = new ApolloClient({
            link: ApolloLink.empty(),
            cache: new InMemoryCache(),
        });
        client.queryManager = {
            resetStore: function () {
                done();
            },
        };
        client.resetStore();
    });
    it('should enable dev tools logging', function () {
        var query = (_a = ["\n      query people {\n        allPeople(first: 1) {\n          people {\n            name\n          }\n        }\n      }\n    "], _a.raw = ["\n      query people {\n        allPeople(first: 1) {\n          people {\n            name\n          }\n        }\n      }\n    "], gql(_a));
        var data = {
            allPeople: {
                people: [
                    {
                        name: 'Luke Skywalker',
                    },
                ],
            },
        };
        it('with self-made store', function () {
            var link = mockSingleLink({
                request: { query: cloneDeep(query) },
                result: { data: data },
            });
            var client = new ApolloClient({
                link: link,
                cache: new InMemoryCache({}, { addTypename: false }),
                addTypename: false,
            });
            var log = [];
            client.__actionHookForDevTools(function (entry) {
                log.push(entry);
            });
            return client.query({ query: query }).then(function () {
                assert.equal(log.length, 2);
                assert.equal(log[1].state.queries['0'].loading, false);
            });
        });
        var _a;
    });
    it('should propagate errors from network interface to observers', function (done) {
        var link = ApolloLink.from([
            function () {
                return new Observable(function (x) {
                    x.error(new Error('Uh oh!'));
                    return;
                });
            },
        ]);
        var client = new ApolloClient({
            link: link,
            cache: new InMemoryCache({}, { addTypename: false }),
            addTypename: false,
        });
        var handle = client.watchQuery({
            query: (_a = ["\n        query {\n          a\n          b\n          c\n        }\n      "], _a.raw = ["\n        query {\n          a\n          b\n          c\n        }\n      "], gql(_a)),
        });
        handle.subscribe({
            error: function (error) {
                assert.equal(error.message, 'Network error: Uh oh!');
                done();
            },
        });
        var _a;
    });
    it('should throw a GraphQL error', function () {
        var query = (_a = ["\n      query {\n        posts {\n          foo\n          __typename\n        }\n      }\n    "], _a.raw = ["\n      query {\n        posts {\n          foo\n          __typename\n        }\n      }\n    "], gql(_a));
        var errors = [
            {
                name: 'test',
                message: 'Cannot query field "foo" on type "Post".',
            },
        ];
        var link = mockSingleLink({
            request: { query: query },
            result: { errors: errors },
        });
        var client = new ApolloClient({
            link: link,
            cache: new InMemoryCache(),
        });
        return client.query({ query: query }).catch(function (err) {
            assert.equal(err.message, 'GraphQL error: Cannot query field "foo" on type "Post".');
        });
        var _a;
    });
    it('should warn if server returns wrong data', function () {
        var query = (_a = ["\n      query {\n        todos {\n          id\n          name\n          description\n          __typename\n        }\n      }\n    "], _a.raw = ["\n      query {\n        todos {\n          id\n          name\n          description\n          __typename\n        }\n      }\n    "], gql(_a));
        var result = {
            data: {
                todos: [
                    {
                        id: '1',
                        name: 'Todo 1',
                        price: 100,
                        __typename: 'Todo',
                    },
                ],
            },
        };
        var link = mockSingleLink({
            request: { query: query },
            result: result,
        });
        var client = new ApolloClient({
            link: link,
            cache: new InMemoryCache(),
        });
        return withWarning(function () { return client.query({ query: query }); }, /Missing field description/);
        var _a;
    });
    it('runs a query with the connection directive and writes it to the store key defined in the directive', function () {
        var query = (_a = ["\n      {\n        books(skip: 0, limit: 2) @connection(key: \"abc\") {\n          name\n        }\n      }\n    "], _a.raw = ["\n      {\n        books(skip: 0, limit: 2) @connection(key: \"abc\") {\n          name\n        }\n      }\n    "], gql(_a));
        var transformedQuery = (_b = ["\n      {\n        books(skip: 0, limit: 2) @connection(key: \"abc\") {\n          name\n          __typename\n        }\n      }\n    "], _b.raw = ["\n      {\n        books(skip: 0, limit: 2) @connection(key: \"abc\") {\n          name\n          __typename\n        }\n      }\n    "], gql(_b));
        var result = {
            books: [
                {
                    name: 'abcd',
                    __typename: 'Book',
                },
            ],
        };
        var link = mockSingleLink({
            request: { query: transformedQuery },
            result: { data: result },
        });
        var client = new ApolloClient({
            link: link,
            cache: new InMemoryCache(),
        });
        return client.query({ query: query }).then(function (actualResult) {
            assert.deepEqual(actualResult.data, result);
        });
        var _a, _b;
    });
    it('should not remove the connection directive at the store level', function () {
        var query = (_a = ["\n      {\n        books(skip: 0, limit: 2) @connection {\n          name\n        }\n      }\n    "], _a.raw = ["\n      {\n        books(skip: 0, limit: 2) @connection {\n          name\n        }\n      }\n    "], gql(_a));
        var transformedQuery = (_b = ["\n      {\n        books(skip: 0, limit: 2) @connection {\n          name\n          __typename\n        }\n      }\n    "], _b.raw = ["\n      {\n        books(skip: 0, limit: 2) @connection {\n          name\n          __typename\n        }\n      }\n    "], gql(_b));
        var result = {
            books: [
                {
                    name: 'abcd',
                    __typename: 'Book',
                },
            ],
        };
        var link = mockSingleLink({
            request: { query: transformedQuery },
            result: { data: result },
        });
        var client = new ApolloClient({
            link: link,
            cache: new InMemoryCache(),
        });
        return client.query({ query: query }).then(function (actualResult) {
            assert.deepEqual(actualResult.data, result);
        });
        var _a, _b;
    });
});
it('should run a query with the connection directive and write the result to the store key defined in the directive', function () {
    var query = (_a = ["\n    {\n      books(skip: 0, limit: 2) @connection(key: \"abc\") {\n        name\n      }\n    }\n  "], _a.raw = ["\n    {\n      books(skip: 0, limit: 2) @connection(key: \"abc\") {\n        name\n      }\n    }\n  "], gql(_a));
    var transformedQuery = (_b = ["\n    {\n      books(skip: 0, limit: 2) @connection(key: \"abc\") {\n        name\n        __typename\n      }\n    }\n  "], _b.raw = ["\n    {\n      books(skip: 0, limit: 2) @connection(key: \"abc\") {\n        name\n        __typename\n      }\n    }\n  "], gql(_b));
    var result = {
        books: [
            {
                name: 'abcd',
                __typename: 'Book',
            },
        ],
    };
    var link = mockSingleLink({
        request: { query: transformedQuery },
        result: { data: result },
    });
    var client = new ApolloClient({
        link: link,
        cache: new InMemoryCache(),
    });
    return client.query({ query: query }).then(function (actualResult) {
        assert.deepEqual(actualResult.data, result);
        assert.deepEqual(client.queryManager.dataStore.getCache().getData(), {
            'ROOT_QUERY.abc.0': { name: 'abcd', __typename: 'Book' },
            ROOT_QUERY: {
                abc: [
                    {
                        generated: true,
                        id: 'ROOT_QUERY.abc.0',
                        type: 'id',
                    },
                ],
            },
        });
    });
    var _a, _b;
});
it('should run a query with the connection directive and filter arguments and write the result to the correct store key', function () {
    var query = (_a = ["\n    query books($order: string) {\n      books(skip: 0, limit: 2, order: $order)\n        @connection(key: \"abc\", filter: [\"order\"]) {\n        name\n        __typename\n      }\n    }\n  "], _a.raw = ["\n    query books($order: string) {\n      books(skip: 0, limit: 2, order: $order)\n        @connection(key: \"abc\", filter: [\"order\"]) {\n        name\n        __typename\n      }\n    }\n  "], gql(_a));
    var result = {
        books: [
            {
                name: 'abcd',
                __typename: 'Book',
            },
        ],
    };
    var variables = { order: 'popularity' };
    var link = mockSingleLink({
        request: { query: query, variables: variables },
        result: { data: result },
    });
    var client = new ApolloClient({
        link: link,
        cache: new InMemoryCache(),
    });
    return client.query({ query: query, variables: variables }).then(function (actualResult) {
        assert.deepEqual(actualResult.data, result);
        assert.deepEqual(client.queryManager.dataStore.getCache().getData(), {
            'ROOT_QUERY.abc({"order":"popularity"}).0': {
                name: 'abcd',
                __typename: 'Book',
            },
            ROOT_QUERY: {
                'abc({"order":"popularity"})': [
                    {
                        generated: true,
                        id: 'ROOT_QUERY.abc({"order":"popularity"}).0',
                        type: 'id',
                    },
                ],
            },
        });
    });
    var _a;
});
function clientRoundtrip(query, data, variables, fragmentMatcher) {
    var link = mockSingleLink({
        request: { query: cloneDeep(query) },
        result: data,
    });
    var client = new ApolloClient({
        link: link,
        cache: new InMemoryCache({}, { fragmentMatcher: fragmentMatcher && fragmentMatcher.match }),
    });
    return client.query({ query: query, variables: variables }).then(function (result) {
        assert.deepEqual(result.data, data.data);
    });
}
//# sourceMappingURL=client.js.map