import { mockObservableLink } from './mocks/mockLinks';
import { assert } from 'chai';
import ApolloClient from '../src';
import gql from 'graphql-tag';
import { QueryManager } from '../src/core/QueryManager';
import { DataStore } from '../src/data/store';
import InMemoryCache from 'apollo-cache-inmemory';
describe('GraphQL Subscriptions', function () {
    var results = [
        'Dahivat Pandya',
        'Vyacheslav Kim',
        'Changping Chen',
        'Amanda Liu',
    ].map(function (name) { return ({ result: { data: { user: { name: name } } }, delay: 10 }); });
    var sub1;
    var options;
    var defaultOptions;
    var defaultSub1;
    beforeEach(function () {
        sub1 = {
            request: {
                query: (_a = ["\n          subscription UserInfo($name: String) {\n            user(name: $name) {\n              name\n            }\n          }\n        "], _a.raw = ["\n          subscription UserInfo($name: String) {\n            user(name: $name) {\n              name\n            }\n          }\n        "], gql(_a)),
                variables: {
                    name: 'Changping Chen',
                },
            },
        };
        options = {
            query: (_b = ["\n        subscription UserInfo($name: String) {\n          user(name: $name) {\n            name\n          }\n        }\n      "], _b.raw = ["\n        subscription UserInfo($name: String) {\n          user(name: $name) {\n            name\n          }\n        }\n      "], gql(_b)),
            variables: {
                name: 'Changping Chen',
            },
        };
        defaultSub1 = {
            request: {
                query: (_c = ["\n          subscription UserInfo($name: String = \"Changping Chen\") {\n            user(name: $name) {\n              name\n            }\n          }\n        "], _c.raw = ["\n          subscription UserInfo($name: String = \"Changping Chen\") {\n            user(name: $name) {\n              name\n            }\n          }\n        "], gql(_c)),
                variables: {
                    name: 'Changping Chen',
                },
            },
        };
        defaultOptions = {
            query: (_d = ["\n        subscription UserInfo($name: String = \"Changping Chen\") {\n          user(name: $name) {\n            name\n          }\n        }\n      "], _d.raw = ["\n        subscription UserInfo($name: String = \"Changping Chen\") {\n          user(name: $name) {\n            name\n          }\n        }\n      "], gql(_d)),
        };
        var _a, _b, _c, _d;
    });
    it('should start a subscription on network interface and unsubscribe', function (done) {
        var link = mockObservableLink(defaultSub1);
        var client = new ApolloClient({
            link: link,
            cache: new InMemoryCache({}, { addTypename: false }),
            addTypename: false,
        });
        var count = 0;
        var sub = client.subscribe(defaultOptions).subscribe({
            next: function (result) {
                count++;
                assert.deepEqual(result, results[0].result);
                if (count > 1) {
                    throw new Error('next fired after unsubscribing');
                }
                sub.unsubscribe();
                done();
            },
        });
        link.simulateResult(results[0]);
    });
    it('should subscribe with default values', function (done) {
        var link = mockObservableLink(sub1);
        var client = new ApolloClient({
            link: link,
            cache: new InMemoryCache({}, { addTypename: false }),
            addTypename: false,
        });
        var count = 0;
        var sub = client.subscribe(options).subscribe({
            next: function (result) {
                assert.deepEqual(result, results[0].result);
                if (count > 1) {
                    throw new Error('next fired after unsubscribing');
                }
                sub.unsubscribe();
                done();
            },
        });
        link.simulateResult(results[0]);
    });
    it('should multiplex subscriptions', function (done) {
        var link = mockObservableLink(sub1);
        var queryManager = new QueryManager({
            link: link,
            store: new DataStore(new InMemoryCache({}, { addTypename: false })),
            addTypename: false,
        });
        var obs = queryManager.startGraphQLSubscription(options);
        var counter = 0;
        obs.subscribe({
            next: function (result) {
                assert.deepEqual(result, results[0].result);
                counter++;
                if (counter === 2) {
                    done();
                }
            },
        });
        obs.subscribe({
            next: function (result) {
                assert.deepEqual(result, results[0].result);
                counter++;
                if (counter === 2) {
                    done();
                }
            },
        });
        link.simulateResult(results[0]);
    });
    it('should receive multiple results for a subscription', function (done) {
        var link = mockObservableLink(sub1);
        var numResults = 0;
        var queryManager = new QueryManager({
            link: link,
            store: new DataStore(new InMemoryCache({}, { addTypename: false })),
            addTypename: false,
        });
        queryManager.startGraphQLSubscription(options).subscribe({
            next: function (result) {
                assert.deepEqual(result, results[numResults].result);
                numResults++;
                if (numResults === 4) {
                    done();
                }
            },
        });
        for (var i = 0; i < 4; i++) {
            link.simulateResult(results[i]);
        }
    });
});
//# sourceMappingURL=graphqlSubscriptions.js.map